/* cplans - fr 2005
   simple critical path plan scheduler
   cplans.c - main, globals and output

   cplans version 0.3  Copyright (C) 2009 Frank Reglin

   cplans is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   cplans is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with cplans; see the file COPYING.
*/

#define CPLANS_GLOBALS
#include "cplans.h"


static void writeoutput( void )
/* ------------------------------------------------------ */
{
int y1, m1, d1, y2, m2, d2, li, emptylines, index, t, buffer, i;

   emptylines = 0;
   for( li = 1; li <= maxlin; li++ )
   {
      index = line[li].idx;
      switch( line[li].kind )
      {
      case EMPTY:
         emptylines++;
         if( emptylines <= MAXEMPTYLINES )
            printf( "\n" );
         break;

      case NOT:
         printf( "note %s\n",  &notes[index] );
         emptylines = 0;
         break;

      case PRJ:
         date2ymd( prj.start, &y1, &m1, &d1 );
         date2ymd( prj.end  , &y2, &m2, &d2 );
         printf( "project start %d-%02d-%02d  end %d-%02d-%02d\n", y1, m1, d1, y2, m2, d2 );
         message( M_INFO2, "duration", strf("%d days", prj.duration) );
         printf( "\n" );
         emptylines = 1;
         break;

      case RES:
         printf( "res %s  use %.3g\n", resource[index].id, resource[index].usable );
         emptylines = 0;
         break;

      case TSK:
         date2ymd( task[index].start, &y1, &m1, &d1 );
         date2ymd( task[index].end  , &y2, &m2, &d2 );
         if( (task[index].res == 0) && (task[index].duration == 0) )
         {
            printf( "task %s  prio %d %s end %d-%02d-%02d  # milestone\n",
               task[index].id, task[index].prio,
               task[index].flags & TASK_FIXED ? "fixed" : "", y2, m2, d2 );
         }
         else if( (task[index].flags & TASK_FIXED) && (task[index].effort == 0) &&
                  (task[index].duration > 0) && (task[index].res != 0) )
         {
            printf( "task %s  prio %d  res %s fixed start %d-%02d-%02d  end %d-%02d-%02d  # vacation\n",
               task[index].id, task[index].prio, resource[task[index].res].id,
               y1, m1, d1, y2, m2, d2 );
            message( M_INFO2, "duration", strf("%d days",task[index].duration) );
         }
         else
         {
            printf( "task %s  prio %d  res %s  effort %d %s start %d-%02d-%02d  end %d-%02d-%02d\n",
               task[index].id, task[index].prio, resource[task[index].res].id, task[index].effort,
               (task[index].flags & TASK_FIXED) || ((task[index].effort == 0) && (task[index].duration > 0)) ?
               "fixed" : "", /* print vacation as fixed here even if the fixed bit was removed */
               y1, m1, d1, y2, m2, d2 );
            message( M_INFO2, "duration", strf("%d days",task[index].duration) );
         }
         if( (infolevel >= 1) && (task[index].flags & TASK_DRAIN) )
         {
            printf( "# %s: predecessors", msgtext[M_INFO1] );
            for( t = 0; t < task[index].numpred; t++ )
               printf( " %s", task[task[index].predecessor[t]].id );
            printf( "\n" );
         }
         if( (infolevel >= 2) && (task[index].flags & TASK_SOURCE) )
         {
            printf( "# %s: successors  ", msgtext[M_INFO2] );
            for( t = 0; t < task[index].numsucc; t++ )
               printf( " %s", task[task[index].successor[t]].id );
            printf( "\n" );
         }
         date2ymd( task[index].eps, &y1, &m1, &d1 );
         date2ymd( task[index].epe, &y2, &m2, &d2 );
         message( M_INFO2, "earliest possible start / end :", strf("%d-%02d-%02d / %d-%02d-%02d",y1,m1,d1,y2,m2,d2) );
         date2ymd( task[index].lps, &y1, &m1, &d1 );
         date2ymd( task[index].lpe, &y2, &m2, &d2 );
         message( M_INFO2, "latest   possible start / end :", strf("%d-%02d-%02d / %d-%02d-%02d",y1,m1,d1,y2,m2,d2) );
         buffer = task[index].lpe - task[index].epe; /* cannot use lps-eps, because eps ignores fixed attribute */
         if( buffer > 0 )
            message( M_INFO1, task[index].id, strf("has %d days buffer\n",buffer) );
         else
            message( M_HINT, task[index].id, "is on the critical path\n" );
         emptylines = 1;
         break;

      case DEP: /* double dependencies have now src and drn = 0, do not print those */
         if( (dependency[index].src != 0) || (dependency[index].drn != 0) )
            printf( "dep %s %s\n", task[dependency[index].src].id, task[dependency[index].drn].id );
         emptylines = 0;
         break;

      default:
         message( M_FATAL, "internal error 1:", strf("line kind %d, line %d",line[li].kind,li) );
      }
   }
   if( !ignorerescoll && (newdep > 0) )
   {
      printf( "# added %d dependencies:\n", newdep );
      for( i = 1; i <= newdep; i++ )
         printf( "dep %s %s\n", task[dependency[maxdep+i].src].id, task[dependency[maxdep+i].drn].id );
   }
}


static void init( void )
/* ------------------------------------------------------ */
{
   errors  = 0;
   maxres  = 0;
   maxtask = 0;
   maxdep  = 0;
   maxlin  = 0;
   nextnote= 0;
   maxprio = 0;
   tempcoll    = 0;
   collentries = 0;
   memset( resource,   0, sizeof(resource  ) );
   memset( task,       0, sizeof(task      ) );
   memset( dependency, 0, sizeof(dependency) );
   memset( line,       0, sizeof(line      ) );
   memset( notes,      0, sizeof(notes     ) );
   memset( &prj,       0, sizeof(prj       ) );

   maxdate = ymd2date( 2099, 12, 31 );

   /* some default values to the "default" resource and task: */
   strcpy( resource[0].id, "NN"    );
   resource[0].usable = 1.0;
   strcpy( task[0].id,     "spare" );
   task[0].prio = -1;

   newdep = 0;
}


static void usage( void )
/* ------------------------------------------------------ */
{
   fprintf( stderr, "USAGE: cplans [options] plan.cps\n" );
   fprintf( stderr, "          options: -f  no fixed tasks\n" );
   fprintf( stderr, "                   -p  no task priorities\n" );
   fprintf( stderr, "                   -r  no resource scheduling\n" );
   fprintf( stderr, "                   -1  take 1st possible variant (not best)\n" );
   fprintf( stderr, "                   -i0 no   infos (default)\n" );
   fprintf( stderr, "                   -i1 some infos\n" );
   fprintf( stderr, "                   -i2 all  infos\n" );
   exit( 1 );
}


int main( int argc, char * argv[] )
/* ------------------------------------------------------ */
{
FILE * inf;
int a, y, m, d;

   fprintf( stderr, "cplans " VERSION "\n\n" );

   if( (argc < 2) || (argc > 6) )
      usage();

   infolevel    = 0;
   ignorefixed  = 0;
   ignoreprio   = 0;
   ignorerescoll= 0;
   take1st      = 0;

   a = 1;
   while( a < argc-1 )
   {
      if( argv[a][0] == '-' )
      {
         switch( argv[a][1] )
         {
         case 'i':
            infolevel = argv[a][2] - '0';
            if( (infolevel < 0) || (infolevel > 2) )
               usage();
            break;
         case 'f':
            ignorefixed  = 1;
            break;
         case 'p':
            ignoreprio   = 1;
            break;
         case 'r':
            ignorerescoll= 1;
            break;
         case '1':
            take1st      = 1;
            break;
         default:
            usage();
         }
      }
      else
         usage();
      a++;
   }

   init();
   inputfn = argv[a];
   inf = fopen( inputfn, "r" );
   if( !inf )
      message( M_FATAL, "cannot open file:", inputfn );

   date2ymd( currentdate(), &y, &m, &d );
   printf( "# made at %d-%02d-%02d by cplans %s\n", y, m, d, VERSION );
   date2ymd( filedate( inputfn ), &y, &m, &d );
   printf( "# input: %s from %d-%02d-%02d\n\n", _fullpath( 0, inputfn, 0 ), y, m ,d );

   readinput( inf );
   plan();
   writeoutput();

   if( errors )
   {
      fprintf( stderr, " %d errors\n", errors );
      exit( 3 );
   }
   exit( 0 );
   return 0;
}

