/* cplans - fr 2005
   simple critical path plan scheduler
   date.c - date handling

   cplans version 0.1  Copyright (C) 2005 Frank Reglin

   cplans is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   cplans is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with cplans; see the file COPYING.
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "date.h"


static int monlen[13] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };


int yearlen( int year )
/* ------------------------------------------------------ */
{
char leap;

   leap = (year % 4) == 0;   // Schaltjahr
   return leap ? 366 : 365;
}


int ymd2date( int year, int month, int day )
/* ------------------------------------------------------ */
{
int date, m, y;
char leap;

   if( year  < 2000 ) return -1;
   if( year  > 2099 ) return -1;
   if( month <    1 ) return -1;
   if( month >   12 ) return -1;
   if( day   <    1 ) return -1;
   if( day   >   31 ) return -1;

   date = 0;
   for( y = 2000; y < year; y++ )
      date += yearlen( y );
   
   leap = (year % 4) == 0;   // Schaltjahr
   if( leap )
      monlen[2] = 29;
   else
      monlen[2] = 28;

   for( m = 1; m < month; m++ )
      date += monlen[m];
   date += day;
   return date;
}


void date2ymd( int date, int * pyear, int * pmonth, int * pday )
/* ------------------------------------------------------ */
{
int y, m, l;
char leap;

   y = 1999;
   for(;;)
   {
      y++;
      l = yearlen(y);
      date -= l;
      if( date <= 0 )
         break;
   }
   date += l;

   leap = (y % 4) == 0;   // Schaltjahr
   if( leap )
      monlen[2] = 29;
   else
      monlen[2] = 28;
   m = 0;
   do
   {
      m++;
      date -= monlen[m];
   } while( date > 0 );
   date += monlen[m];
   *pyear = y;
   *pmonth = m;
   *pday = date;
}


int time2date( time_t time )
/* ------------------------------------------------------ */
{
static struct tm t;
time_t t0;
int seconds, date;

   t.tm_sec  = 0;
   t.tm_min  = 0;
   t.tm_hour = 0;
   t.tm_mday = 1;
   t.tm_mon  = 0;
   t.tm_year = 2000-1900;
   t0 = mktime( &t ); // (UNIX) time at Jan 0 2000

   seconds = time - t0; // seconds since Jan 0 2000
   date = (seconds + (60*60*24) -1 ) / (60*60*24);
   return date;
}


int currentdate( void )
/* ------------------------------------------------------ */
{
   return time2date( time( 0 ) );
}


int filedate( char * filename )
/* ------------------------------------------------------ */
{
static struct stat s;
int rc;

   rc = stat( filename, &s );
   if( rc != 0 )
      return -1; // error

   return time2date( s.st_mtime );
}


int atodate( char * str )
/* ------------------------------------------------------ */
/* returns -1 on error */
{
int y, m, d, k;

   k = sscanf( str, "%d-%d-%d", &y, &m, &d );
   if( k != 3 )
      return -1; /* error */
   return ymd2date( y, m, d );
}


