/* cplans - fr 2005
   simple critical path plan scheduler
   util.c - utility functions

   cplans version 0.1  Copyright (C) 2005 Frank Reglin

   cplans is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   cplans is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with cplans; see the file COPYING.
*/

#include "cplans.h"

#ifdef LINUX
#include <unistd.h>
#else
#define isatty _isatty
#endif

char * msgtext[6] =
{
   "FATAL", "ERROR", "WARNING", "hint", "info", "info"
};



void message( int kind, char * txt1, char * txt2 )
/* ------------------------------------------------------ */
{
   if( (infolevel+M_INFO1) > kind )
      printf( "# %s: %s %s\n", msgtext[kind], txt1, txt2 );
   if( (kind <= M_WARNING) && !isatty( 1 /*STDOUT*/ ) )
      fprintf( stderr, "%s: %s %s\n", msgtext[kind], txt1, txt2 );

   if( kind <= M_ERROR )
      errors++;
   if( kind == M_FATAL )
      exit( 2 );
}


char * itostr( int i )
/* ------------------------------------------------------ */
{
static char buf[100];

   sprintf( buf, "%d", i );
   return buf;
}


char * strf( char * format, ... )
/* ------------------------------------------------------ */
{
static char buffer[MAXSTRINGLEN];
va_list ap;

   va_start( ap, format );
   vsprintf( buffer, format, ap );
   va_end( ap );
   return buffer;
}


int imin( int a, int b )
/* ------------------------------------------------------ */
{
   return a <= b ? a : b;
}


int imax( int a, int b )
/* ------------------------------------------------------ */
{
   return a >= b ? a : b;
}


void * alloc( int size )
/* ------------------------------------------------------ */
{
void * ptr;

   ptr = calloc( (size+3)/4, 4 );
   if( ptr == 0 )
      message( M_FATAL, "not enough memory", "" );

   return ptr;
}


int nextperm( short * a, int num )
/* ------------------------------------------------------ */
/* bestimmt eine Permutation pro Aufruf
   num Werte im Feld a, anfangs der Groesse nach sortiert,
   aber nicht unbedingt aufeinanderfolgend
   returns 0 iff dieser Aufruf wieder den
   Anfangszustand herstellt. ("eine Runde 'rum" )         */
{
int i;
short exch;

   if( num < 2 )
      return 0;   /* sollte garnicht vorkommen */

   if( num == 2 )
   {
      exch = a[1];
      a[1] = a[0];
      a[0] = exch;
      return a[0] > a[1];
   }
   else
   {
      if( nextperm( a + 1, num - 1 ) )
         return 1;

      /* ok, a[1] bis a[num-1] sind nun wieder der Groesse nach sortiert und
         a[0] muss einen groesseren Wert bekommen, wenn es nicht schon den
         groessten hat...                                                    */

      for( i = 1; i < num; i++ )
      {
         if( a[i] > a[0] )
         {         /* naechsten Wert fuer a[0] gefunden, tauschen: */
            exch = a[i];
            a[i] = a[0];
            a[0] = exch;
            return 1;
         }
      }

      /* a[0] hatte schon den Maximalwert, d.h. die letzte Permutation ist
         vorbei. Kleinsten Wertr wieder an den Anfang, dies sollte a[1]
         sein. a[0] muss and Ende:                                        */
      exch = a[0];
      for( i = 1; i < num; i++ )
         a[i-1] = a[i];
      a[num-1] = exch;
      return 0;    /* fertig */
   }
}


int fak( int n )
/* ------------------------------------------------------ */
/* n! */
{
int i, nf;

   if( n <= 1 )
      return 1;

   if( n > 12 )
      return -1; /* overflow */

   nf = 1;
   for( i = 2; i <= n; i++ )
   {
      nf *= i;
   }

   return nf;
}


